// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

class TestReporter {
  start(TestSuite suite);

  begin(Function test);
  pass(Function test);
  fail(Function test, ExpectException failedExpectation);
  skip(Function test, AssumeException failedAssumption);
  error(Function test, exception);

  fatal(exception);

  end();
}

class NullReporter implements TestReporter {
  start(TestSuite suite) {}

  begin(Function test) {}
  pass(Function test) {}
  fail(Function test, ExpectException failedExpectation) {}
  skip(Function test, AssumeException failedAssumption) {}
  error(Function test, exception) {}

  fatal(exception) {}

  end() {}
}

class ConsoleReporter implements TestReporter {
  static final _FUNCTION_NAME_PATTERN = const RegExp("from Function '(.*?)':");

  int counter = 0;
  int total = 0;

  int passed = 0;
  int failed = 0;
  int skipped = 0;
  int errors = 0;

  int fatals = 0;

  start(TestSuite suite) {
    total += suite.tests.length; // can be called many times!

    print(suite.name);
    var dashes = new StringBuffer();
    for (int i = 0; i < suite.name.length; i++) {
      dashes.add('─');
    }
    print("$dashes");
  }

  _testName(Function test) {
    var functionName = _FUNCTION_NAME_PATTERN.firstMatch("$test");
    if (functionName != null) {
      return functionName[1];
    }
    return "<test no. $counter>";
  }

  _exceptionDescription(e) => "$e".replaceAll("\n", "\n  ");

  begin(Function test) {
    counter++;
    stdout.writeString(white('> ${_testName(test)}'));
  }

  pass(Function test) {
    passed++;
    stdout.writeString(lineBegin);
    print(lightGreen("✔ ${_testName(test)}"));
  }

  fail(Function test, ExpectException failedExpectation) {
    failed++;
    stdout.writeString(lineBegin);
    print(lightRed("✘ ${_testName(test)}"));
    print(gray("  ${_exceptionDescription(failedExpectation)}"));
  }

  skip(Function test, AssumeException failedAssumption) {
    skipped++;
    stdout.writeString(lineBegin);
    print(lightBlue("∅ ${_testName(test)}"));
    print(gray("  ${_exceptionDescription(failedAssumption)}"));
  }

  error(Function test, exception) {
    errors++;
    stdout.writeString(lineBegin);
    print(yellow("‼ ${_testName(test)}"));
    print(gray("  ${_exceptionDescription(exception)}"));
  }

  fatal(exception) {
    fatals++;
    stdout.writeString(lineBegin);
    print(lightMagenta("✞ ${_exceptionDescription(exception)}"));
  }

  _count(int count) => count == 1 ? "1 test" : "$count tests";

  end() {
    var fatalsDesc = fatals == 0 ? "" : ", $fatals fatal error${fatals > 1 ? "s" : ""}";

    print("");
    print("${_count(counter)} out of total ${_count(total)}$fatalsDesc");
    print("${_count(passed)} passed, ${_count(failed)} failed, ${_count(skipped)} skipped, ${_count(errors)} in error");
  }
}

class MultipleSuitesAwareReporter implements TestReporter {
  final TestReporter delegate;

  MultipleSuitesAwareReporter(this.delegate);

  start(suite) => delegate.start(suite);
  begin(test) => delegate.begin(test);
  pass(test) => delegate.pass(test);
  fail(test, failedExpectation) => delegate.fail(test, failedExpectation);
  skip(test, failedAssumption) => delegate.skip(test, failedAssumption);
  error(test, exception) => delegate.error(test, exception);
  fatal(exception) => delegate.fatal(exception);
  end() {} // not yet

  finish() => delegate.end();
}

