// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

class _Table implements Iterable {
  List<String> _header;
  Function _transformation;
  List _rows;
  List _currentRow;

  _Table() : _header = [], _rows = [], _currentRow = [];

  operator *(String columnName) {
    _header.add(columnName);
    return this;
  }

  operator >(Function transformation) {
    _transformation = transformation;
    return this;
  }

  operator |(value) {
    _currentRow.add(value);
    if (_currentRow.length == _header.length) {
      var row = new _Row(_header, _currentRow);
      _rows.add(_transformation == null ? row : _transformation(row));
      _currentRow = [];
    }
    return this;
  }

  Iterator iterator() => _rows.iterator();
}

class _Row implements Iterable {
  List _cells;
  Map<String, Dynamic> _cellsMap;

  _Row(List<String> header, List values) {
    if (header.length != values.length) {
      throw new DslException("Table row $values has bad number of columns (table header: $header)");
    }

    _cells = values;
    _cellsMap = new LinkedHashMap<String, Dynamic>();
    for (int i = 0; i < header.length; i++) {
      _cellsMap[header[i]] = values[i];
    }
  }

  operator [](index) {
    if (index is int) {
      return _cells[index];
    } else if (index is String) {
      return _cellsMap[index];
    } else {
      throw new DslException("Table row can only be indexed by an 'int' or a 'String', but got '$index'");
    }
  }

  noSuchMethod(String name, List arguments) {
    if (name.startsWith("get:")) {
      var property = name.substring(4);
      if (_cellsMap.containsKey(property)) {
        return _cellsMap[property];
      } else {
        throw new DslException("Table row doesn't contain '$property'");
      }
    }
    super.noSuchMethod(name, arguments);
  }

  Iterator iterator() => _cells.iterator();

  int get length() => _cells.length;

  toString() => "$_cellsMap";
}

class _Expectation {
  final int number;
  final expectation;
  final String description;

  _Expectation(this.number, this.expectation, this.description) {
    if (expectation is! Matcher
        && expectation is! Predicate
        && expectation is! BoolFunction
        && expectation is! bool) {
      throw new DslException("Expectation can only be a 'Matcher', 'Predicate',"
          "'BoolFunction' or 'bool', but got '$expectation'");
    }
  }

  verify(value) {
    var message = description != null ? "expectation failed: $description"
        : "expectation no. $number failed";

    if (expectation is Matcher) {
      expectThat(value, expectation, message);
    } else if (expectation is Predicate) {
      bool result = expectation(value);
      if (!result) {
        throw new ExpectException(message);
      }
    } else if (expectation is BoolFunction) {
      if (!expectation()) {
        throw new ExpectException(message);
      }
    } else if (expectation is bool) {
      if (!expectation) {
        throw new ExpectException(message);
      }
    }
  }
}

class _Expectations {
  List<_Expectation> _expectations;
  int _counter;
  String _nextDescription;

  _Expectations() : _expectations = [], _counter = 1;

  operator *(description) {
    if (description is String) {
      _nextDescription = description;
    } else { // description isn't mandatory, it can be an expectation
      this / description;
    }
    return this;
  }

  operator /(expectation) {
    var expectationObj = new _Expectation(_counter, expectation, _nextDescription);
    _counter++;
    _nextDescription = null;

    if (expectation is bool || expectation is BoolFunction) { // verify eagerly
      expectationObj.verify(null); // value doesn't make sense in this case
    } else {
      _expectations.add(expectationObj);
    }
    return this;
  }

  operator <(value) {
    for (final expectation in _expectations) {
      if (value is _Each) {
        for (final element in value._iterable) {
          expectation.verify(element);
        }
      } else {
        expectation.verify(value);
      }
    }
  }
}

class _Each {
  final Iterable _iterable;

  _Each(this._iterable);
}

